package com.sample.processserver;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SystemEventListenerFactory;
import org.drools.base.MapGlobalResolver;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.compiler.BPMN2ProcessFactory;
import org.drools.compiler.ProcessBuilderFactory;
import org.drools.io.ResourceFactory;
import org.drools.marshalling.impl.ProcessMarshallerFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessRuntimeFactory;
import org.jbpm.bpmn2.BPMN2ProcessProviderImpl;
import org.jbpm.marshalling.impl.ProcessMarshallerFactoryServiceImpl;
import org.jbpm.process.builder.ProcessBuilderFactoryServiceImpl;
import org.jbpm.process.instance.ProcessRuntimeFactoryServiceImpl;
import org.jbpm.task.AccessType;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.ContentData;
import org.jbpm.task.service.TaskClient;
import org.jbpm.task.service.mina.MinaTaskClientConnector;
import org.jbpm.task.service.mina.MinaTaskClientHandler;
import org.jbpm.task.service.responsehandlers.BlockingTaskOperationResponseHandler;
import org.jbpm.task.service.responsehandlers.BlockingTaskSummaryResponseHandler;

import bitronix.tm.TransactionManagerServices;

public class JbpmAPIUtil
{
  private static String ipAddress = "127.0.0.1";

  //private static int port = 5445;
  private static int port = 9123;
  private static TaskClient client;
  private static Map<String, List<String>> groupListMap = new HashMap();
  private static StatefulKnowledgeSession ksession;

  public void setConnection(String ipAddress, int port)
  {
    ipAddress = ipAddress;
    port = port;
  }

  public static void connect()
  {
    if (client == null) {
      client = new TaskClient(
        new MinaTaskClientConnector("org.drools.process.workitem.wsht.WSHumanTaskHandler", 
        new MinaTaskClientHandler(SystemEventListenerFactory.getSystemEventListener())));
      boolean connected = client.connect(ipAddress, port);
      if (!connected)
        throw new IllegalArgumentException(
          "Could not connect task client");
    }
    try
    {
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      URL url = null;
      String propertyName = "roles.properties";

      if ((loader instanceof URLClassLoader)) {
        URLClassLoader ucl = (URLClassLoader)loader;
        url = ucl.findResource(propertyName);
      }
      if (url == null) {
        url = loader.getResource(propertyName);
      }
      if (url == null) {
        System.out.println("No properties file: " + propertyName + " found");
      } else {
        Properties bundle = new Properties();
        InputStream is = url.openStream();
        if (is != null) {
          bundle.load(is);
          is.close();
        } else {
          throw new IOException("Properties file " + propertyName + " not available");
        }
        Enumeration propertyNames = bundle.propertyNames();
        while (propertyNames.hasMoreElements()) {
          String key = (String)propertyNames.nextElement();
          String value = bundle.getProperty(key);
          groupListMap.put(key, Arrays.asList(value.split(",")));
          System.out.print("Loaded user " + key + ":");
          for (String role : (List<String>)groupListMap.get(key)) {
            System.out.print(" " + role);
          }
          System.out.println();
        }
      }
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }

  public static List<TaskSummary> getAssignedTasks(String idRef)
  {
    connect();
    List tasks = null;
    try {
      BlockingTaskSummaryResponseHandler responseHandler = new BlockingTaskSummaryResponseHandler();
      client.getTasksOwned(idRef, "en-UK", responseHandler);
      tasks = responseHandler.getResults();
    } catch (Throwable t) {
      t.printStackTrace();
    }
    return tasks;
  }

  public static void completeTask(long taskId, Map data, String userId)
    throws InterruptedException
  {
    connect();

    BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();
    client.start(taskId, userId, responseHandler);
    responseHandler.waitTillDone(5000L);

    responseHandler = new BlockingTaskOperationResponseHandler();
    ContentData contentData = null;
    if (data != null) {
      ByteArrayOutputStream bos = new ByteArrayOutputStream();
      try
      {
        ObjectOutputStream out = new ObjectOutputStream(bos);
        out.writeObject(data);
        out.close();
        contentData = new ContentData();
        contentData.setContent(bos.toByteArray());
        contentData.setAccessType(AccessType.Inline);
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
    client.complete(taskId, userId, contentData, responseHandler);
    responseHandler.waitTillDone(5000L);
  }

  public static void completeProgressTask(long taskId, Map data, String userId)
    throws InterruptedException
  {
    connect();

    BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();
    responseHandler.waitTillDone(5000L);

    responseHandler = new BlockingTaskOperationResponseHandler();
    ContentData contentData = null;
    if (data != null) {
      ByteArrayOutputStream bos = new ByteArrayOutputStream();
      try
      {
        ObjectOutputStream out = new ObjectOutputStream(bos);
        out.writeObject(data);
        out.close();
        contentData = new ContentData();
        contentData.setContent(bos.toByteArray());
        contentData.setAccessType(AccessType.Inline);
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
    client.complete(taskId, userId, contentData, responseHandler);
    responseHandler.waitTillDone(5000L);
  }

  public void assignTask(long taskId, String idRef, String userId)
  {
    connect();
    BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();
    if (idRef == null) {
      client.release(taskId, userId, responseHandler);
    } else if (idRef.equals(userId)) {
      List roles = (List)groupListMap.get(userId);
      if (roles == null)
        client.claim(taskId, idRef, responseHandler);
      else
        client.claim(taskId, idRef, roles, responseHandler);
    }
    else {
      client.delegate(taskId, userId, idRef, responseHandler);
    }
    responseHandler.waitTillDone(5000L);
  }

  public static KnowledgeBase readKnowledgeBase(String process)
    throws Exception
  {
    ProcessBuilderFactory.setProcessBuilderFactoryService(new ProcessBuilderFactoryServiceImpl());
    ProcessMarshallerFactory.setProcessMarshallerFactoryService(new ProcessMarshallerFactoryServiceImpl());
    ProcessRuntimeFactory.setProcessRuntimeFactoryService(new ProcessRuntimeFactoryServiceImpl());
    BPMN2ProcessFactory.setBPMN2ProcessProvider(new BPMN2ProcessProviderImpl());
    KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
    kbuilder.add(ResourceFactory.newClassPathResource(process), ResourceType.BPMN2);
    return kbuilder.newKnowledgeBase();
  }

  public static StatefulKnowledgeSession getSession()
    throws Exception
  {
    if (ksession == null) {
      ksession = createSession();
    }
    return ksession;
  }

  public static StatefulKnowledgeSession createSession()
    throws Exception
  {
    System.out.print("ok1");
    KnowledgeBase kbase = readKnowledgeBase("SampleHumanTaskFormVariables.bpmn");
    System.out.print("ok2");
    EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa");
    System.out.print("ok3");
    Environment env = KnowledgeBaseFactory.newEnvironment();
    env.set("drools.persistence.jpa.EntityManagerFactory", emf);
    env.set("drools.transaction.TransactionManager", TransactionManagerServices.getTransactionManager());
    env.set("drools.Globals", new MapGlobalResolver());

    Properties properties = new Properties();
    properties.put("drools.processInstanceManagerFactory", "org.jbpm.persistence.processinstance.JPAProcessInstanceManagerFactory");
    properties.put("drools.processSignalManagerFactory", "org.jbpm.persistence.processinstance.JPASignalManagerFactory");
    KnowledgeSessionConfiguration config = KnowledgeBaseFactory.newKnowledgeSessionConfiguration(properties);

    return JPAKnowledgeService.newStatefulKnowledgeSession(kbase, config, env);
  }
}